--[[
    Title: PlayerTarget
    Author: Grimmier
    Version:0.5
    Description: Combines Player Information window and Target window into one.
    Displays Your player info. as well as Target: Hp, Your aggro, SecondaryAggroPlayer, Visability, Distance,
    and Buffs with name \ duration on tooltip hover.
]]
---@type Mq
local mq = require('mq')
---@type ImGui
local ImGui = require('ImGui')
local Icons = require('mq.ICONS')
-- set variables
local anim = mq.FindTextureAnimation('A_SpellIcons')
local winFlag = bit32.bor(ImGuiWindowFlags.NoTitleBar)
local function barColor(c)
    if (c == 'red') then return ImGui.PushStyleColor(ImGuiCol.PlotHistogram,.7, 0, 0, .7) end
    if (c == 'blue') then return ImGui.PushStyleColor(ImGuiCol.PlotHistogram, .2, .6, 1, .4) end
    if (c == 'yellow') then return ImGui.PushStyleColor(ImGuiCol.PlotHistogram, .7, .6, .1, .7) end
    if (c == 'purple') then return ImGui.PushStyleColor(ImGuiCol.PlotHistogram, .6, .0, .6, .7) end
    if (c == 'grey') then return ImGui.PushStyleColor(ImGuiCol.PlotHistogram, 1, 1, 1, .2) end
end
local function getDuration(i)
    local remaining = mq.TLO.Target.Buff(i).Duration() or 0
    remaining = remaining / 1000 -- convert to seconds
    -- Calculate hours, minutes, and seconds
    local h = math.floor(remaining / 3600) or 0
    remaining = remaining % 3600 -- remaining seconds after removing hours
    local m = math.floor(remaining / 60) or 0
    local s = remaining % 60 -- remaining seconds after removing minutes
    -- Format the time string as H : M : S
    local sRemaining = string.format("%02d:%02d:%02d", h, m, s)
    return sRemaining
end
local function targetBuffs(count)
    -- Save the original item spacing
    local originalSpacingX, originalSpacingY = ImGui.GetStyle().ItemSpacing.x, ImGui.GetStyle().ItemSpacing.y
    ImGui.PushStyleVar(ImGuiStyleVar.ItemSpacing, 0, 0)
    -- Width and height of each texture
    local textureWidth = 20
    local textureHeight = 20
    local iconsDrawn = 0
    -- Calculate max icons per row based on the window width
    local windowWidth = ImGui.GetWindowContentRegionWidth()
    local maxIconsRow = math.floor(windowWidth / (textureWidth))
    ImGui.BeginGroup()
    for i = 1, count do
        local sIcon = mq.TLO.Target.Buff(i).SpellIcon()
        anim:SetTextureCell(sIcon)
        ImGui.DrawTextureAnimation(anim, textureWidth, textureHeight)
        if ImGui.IsItemHovered() then
            ImGui.BeginTooltip()
            ImGui.Text(mq.TLO.Target.Buff(i).Name()..' '..getDuration(i))
            ImGui.EndTooltip()
        end
        iconsDrawn = iconsDrawn + 1
        -- Check if we've reached the max icons for the row, if so reset counter and new line
        if iconsDrawn >= maxIconsRow then
            iconsDrawn = 0 -- Reset counter
            else
            -- Use SameLine to keep drawing items on the same line, except for when a new line is needed
            if i < count then
                ImGui.SameLine()
            end
        end
    end
    ImGui.EndGroup()
    ImGui.PopStyleVar()
end
function GUI_Target(open)
    ImGui.PushStyleVar(ImGuiStyleVar.FrameRounding, 5)
    -- change the window size
    ImGui.SetNextWindowSize(300, 300, ImGuiCond.FirstUseEver)
    local show = false
    open, show = ImGui.Begin("Target", open, winFlag)
    if not show then
        ImGui.End()
        return open
    end
    ImGui.SetWindowFontScale(1)
    ImGui.Text(mq.TLO.Me.Name()..'\n'..mq.TLO.Me.Class())
    ImGui.SameLine(ImGui.GetWindowWidth() - 40)
    ImGui.Text(mq.TLO.Me.Level())
    ImGui.SetWindowFontScale(0.75)
    barColor('red')
    ImGui.ProgressBar(((tonumber(mq.TLO.Me.PctHPs() or 0))/100), ImGui.GetContentRegionAvail(), 10)
    ImGui.PopStyleColor()
    barColor('blue')
    if (tonumber(mq.TLO.Me.MaxMana())>0) then
        ImGui.ProgressBar(((tonumber(mq.TLO.Me.PctMana() or 0))/100), ImGui.GetContentRegionAvail(), 10)
        else
        ImGui.Text('')
    end
    ImGui.PopStyleColor()
    barColor('yellow')
    ImGui.ProgressBar(((tonumber(mq.TLO.Me.PctEndurance() or 0))/100), ImGui.GetContentRegionAvail(), 10)
    ImGui.PopStyleColor()
    ImGui.Separator()
    if (mq.TLO.Target()~= nil) then
        ImGui.SetWindowFontScale(0.9)
        ImGui.Text(mq.TLO.Target.CleanName())
        ImGui.Text(mq.TLO.Target.Level())
        ImGui.SameLine()
        ImGui.Text(mq.TLO.Target.Race())
        ImGui.SameLine(ImGui.GetWindowWidth()/2)
        ImGui.Text(mq.TLO.Target.Class.ShortName())
        ImGui.SameLine(ImGui.GetWindowWidth() - 40)
        ImGui.Text(tostring(math.floor(mq.TLO.Target.Distance() or 0)))
        ImGui.SetWindowFontScale(0.75)
        barColor('red')
        ImGui.ProgressBar(((tonumber(mq.TLO.Target.PctHPs()or 0))/100), ImGui.GetContentRegionAvail(), 10)
        ImGui.PopStyleColor()
        if (mq.TLO.Target.Aggressive) then
            barColor('purple')
            ImGui.ProgressBar(((tonumber(mq.TLO.Target.PctAggro() or 0))/100), ImGui.GetContentRegionAvail(), 10)
            ImGui.PopStyleColor()
            else
            ImGui.Text('')
        end
        if (mq.TLO.Target.AggroHolder()~= nil) then
            ImGui.SetWindowFontScale(0.9)
            local agH = mq.TLO.Target.SecondaryAggroPlayer()
            ImGui.Text(agH)
            ImGui.SameLine(ImGui.GetWindowWidth()/2)
            ImGui.SetWindowFontScale(1.1)
            if (mq.TLO.Target.LineOfSight) then
                ImGui.PushStyleColor(ImGuiCol.Text, 0, 1, 0, 1)
                ImGui.Text(Icons.MD_VISIBILITY)
                ImGui.PopStyleColor()
                else
                ImGui.PushStyleColor(ImGuiCol.Text, 0.9, 0, 0, 1)
                ImGui.Text(Icons.MD_VISIBILITY_OFF)
                ImGui.PopStyleColor()
            end
            ImGui.SetWindowFontScale(0.9)
            ImGui.SameLine(ImGui.GetWindowWidth()-40)
            ImGui.Text(mq.TLO.Target.SecondaryPctAggro())
            else
            ImGui.Text('')
            ImGui.SameLine(ImGui.GetWindowWidth()/2)
            ImGui.SetWindowFontScale(1.1)
            if (mq.TLO.Target.LineOfSight()) then
                ImGui.PushStyleColor(ImGuiCol.Text, 0, 1, 0, 1)
                ImGui.Text(Icons.MD_VISIBILITY)
                ImGui.PopStyleColor()
                else
                ImGui.PushStyleColor(ImGuiCol.Text, 0.9, 0, 0, 1)
                ImGui.Text(Icons.MD_VISIBILITY_OFF)
                ImGui.PopStyleColor()
            end
        end
        ImGui.Separator()
        if tonumber(mq.TLO.Target.BuffCount()) > 0 then
            targetBuffs(tonumber(mq.TLO.Target.BuffCount()))
        end
        else
        ImGui.Text('')
    end
    ImGui.PopStyleVar()
    ImGui.Spacing()
    ImGui.End()
    return open
end
local openGUI = true
ImGui.Register('changeMe', function()
    openGUI = GUI_Target(openGUI)
end)
while openGUI do
    mq.delay(1000)
end